#include <sort.h>

#include <stdlib.h> // malloc, free
#include <stdio.h> // fprintf
#include <time.h> // clock_gettime
#include <string.h> //strcmp

#define BILLION 1000000000L

const long DEFAULT_SIZE = 50000000;
long SIZE;

const int DEFAULT_NUM_THREADS = 1;
int numThreads;

TYPE * generateArrayToSort() {
  TYPE * array = (TYPE *) malloc(sizeof(TYPE) * SIZE);
  srand(0);
  int i;
  for(i = 0; i < SIZE; i++)
    array[i] = rand();
  return array;
}

void print(TYPE * const array, int size) {
  int i;
  for (i = 0; i < size; i++) {
    fprintf(stdout, "%d\n", array[i]);
  }
}

void USAGE(const char * s) {
  fprintf(stderr, "USAGE: %s -t [num threads] -n [number of array elements]\n", s);
}

int main(int argc, char * argv[]) {
  int i;
  char numThreadsSpecified = 0, sizeSpecified = 0;
  for (i = 1; i < argc; i++) {
    if (strcmp(argv[i], "-t") == 0) {
      if ((i + 1) == argc) {
        fprintf(stderr, "Number of threads unspecified\n");
        return EXIT_FAILURE;
      }
      numThreads = atoi(argv[++i]);
      numThreadsSpecified = 1;
    } else if (strcmp(argv[i], "-n") == 0) {
      if ((i + 1) == argc) {
        fprintf(stderr, "Size of array unspecified\n");
        return EXIT_FAILURE;
      }
      SIZE = atoi(argv[++i]);
      sizeSpecified = 1;
    } else if ((strcmp(argv[i], "-h") == 0) || (strcmp(argv[i], "--help") == 0)) {
      USAGE(argv[0]);
      return EXIT_FAILURE;
    } else {
      USAGE(argv[0]);
      return EXIT_FAILURE;
    }
  }

  if (!numThreadsSpecified) numThreads = DEFAULT_NUM_THREADS;
  if (!sizeSpecified) SIZE = DEFAULT_SIZE;

  if (numThreads < 1) {
    fprintf(stderr, "Number of threads must be greater than zero\n");
    if (argc >= 2) {
      fprintf(stderr, "Got num threads = %d\n", numThreads);
    }
    return EXIT_FAILURE;
  }
  if (SIZE < 1) {
    fprintf(stderr, "Array size must be greater than zero\n");
    if (argc >= 3) {
      fprintf(stderr, "Got array size = %ld\n", SIZE);
    }
    return EXIT_FAILURE;
  }

  TYPE * const array = (TYPE * const) generateArrayToSort();

  struct timespec start, stop;
  double totalTime;

  if(clock_gettime(CLOCK_REALTIME, &start) == -1) {
    perror("clock gettime");
    return EXIT_FAILURE;
  }

  sort(array, 0, SIZE - 1);

  if(clock_gettime(CLOCK_REALTIME, &stop) == -1) {
    perror("clock gettime");
    return EXIT_FAILURE;
  }

  totalTime = (stop.tv_sec - start.tv_sec)
    + (double)(stop.tv_nsec - start.tv_nsec)
    / (double)BILLION;
  fprintf(stdout, "%lf\n", totalTime);

  free(array);

  return EXIT_SUCCESS;
}
